/* ***** BEGIN LICENSE BLOCK *****
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is: YSlow 2
 *
 * The Initial Developer of the Original Code is Yahoo! Inc.
 *
 * Copyright (C) 2009, Yahoo! Inc. All Rights Reserved.
 *
 * Contributor(s):
 *     Antonia Kwok
 *
 * ***** END LICENSE BLOCK *****/

/**
 * This class implements Firefox's nsIWebProgressListener.
 * It listens to tab progress, especially onLocationChange.
 *
 * @class
 */
var tabProgressListener =
{
    QueryInterface : function(iid)
    {
        if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
            iid.equals(Components.interfaces.nsISupportsWeakReference) ||
            iid.equals(Components.interfaces.nsISupports))
        {
            return this;
        }

        throw Components.results.NS_NOINTERFACE;
    },

    stateIsRequest: false,
    onLocationChange: function(progress, request, uri)
    {
        // Only watch windows that are their own parent - e.g. not frames
        if (progress.DOMWindow.parent == progress.DOMWindow)
        {
            YSLOW.firefox.watchTopWindow(progress.DOMWindow);
        }
    },
    onStateChange : function() {},
    onProgressChange : function() {},
    onStatusChange : function() {},
    onSecurityChange : function() {},
    onLinkIconAvailable : function() {}
};


/**
 * Firefox integration
 *
 * @namespace YSLOW
 * @class firefox
 * @static
 */

YSLOW.firefox = {

    http_observer: null,

    panel: null,

    init: function() {

        // register networking support from Firefox
        YSLOW.net.registerNative(YSLOW.firefox.net);

        if (YSLOW.util.Preference.getPref("extensions.yslow.observeNetwork", true) &&
            (!this.http_observer || typeof this.http_observer == "undefined")) {
            this.http_observer = new YSLOW.firefox.observer();
        }
        window.addEventListener("load", function(event) {
                                    gBrowser.addProgressListener(tabProgressListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
                                }, false);

        YSLOW.controller.init();

        // add yslow event listeners.
        YSLOW.util.event.addListener("peelStart", this.onPeelStart, this);
        YSLOW.util.event.addListener("peelProgress", this.showPeelProgress, this);
        YSLOW.util.event.addListener("componentFetchProgress", this.showFetchProgress, this);
        YSLOW.util.event.addListener("componentFetchDone", this.onFetchDone, this);

        YSLOW.view.toggleStatusBar(YSLOW.util.Preference.getPref("extensions.yslow.hidestatusbar"));
    },

    startup: function(wmode) {
        this.windowMode = wmode;

        // Preference branch has been changed starting YSlow2, copy over the preference from YSlow1
        var prefs = YSLOW.util.Preference.getPrefList("extensions.firebug.yslow.");
        if (prefs && prefs.length > 0) {
            for (var i = 0; i < prefs.length; i++) {
                if (YSLOW.util.Preference.getPref(prefs[i].name) !== undefined) {
                    YSLOW.util.Preference.setPref(prefs[i].name, prefs[i].value);
                }
                // delete old pref
                YSLOW.util.Preference.deletePref("extensions.firebug.yslow." + prefs[i].name);
            }
        }

        if (typeof FirebugContext !== "undefined") {
            try {
            if (Firebug === null) {
                FBL.ns(function() {
                           YSLOW.FBYSlow.init();
                       });
            } else {
                YSLOW.FBYSlow.init();
            }
            } catch(err) {
                YSLOW.util.dump("YSLOW.firefox.startup():" + err);
            }
            return;
        }

        YSLOW.firefox.init();
    },

    shutdown: function() {

        if (!this.http_observer || typeof this.http_observer == "undefined") {
            this.http_observer.unregister();
            this.http_observer = null;
        }

        gBrowser.removeProgressListener(tabProgressListener);
        for (var i = 0; i < gBrowser.browsers.length; i++) {
            var browser = gBrowser.browsers[i];
            YSLOW.firefox.unwatchTopWindow(browser.contentWindow);
        }

    },

    close: function() {

	var panel = document.getElementById("yslow-content-box");
	var splitter = document.getElementById("yslow-content-splitter");
	splitter.collapsed = panel.collapsed = true;

    },

    run: function(autorun) {

        if (typeof FirebugContext !== "undefined") {
            Firebug.YSlow.run(autorun);
            return;
        }

        var context;
        if (this.panel && this.panel.yslowContext) {
            this.panel.getPanel();
            context = this.panel.yslowContext;
        } else {
            context = new YSLOW.context(window.top.content.document);
        }
        YSLOW.controller.run(this.win, context, autorun);

    },

    watchBrowser: function(browser) {
        return this.watchTopWindow(browser.contentWindow);
    },

    unwatchBrowser: function(browser) {
        return this.unwatchTopWindow(browser.contentWindow);
    },

    watchLoadedTopWindow: function(win) {
        var t_start;
        var now = Number(new Date());

        YSLOW.view.clearStatusBar();

        var yslow_context;
        if (this.panel) {
            t_start = this.panel.browser.t_start;
            this.panel.getPanel();
            if (this.panel.yslowContext) {
                yslow_context = this.panel.yslowContext;
            } else {
                yslow_context = new YSLOW.context(window.top.content.document);
            }
            yslow_context.PAGE.loaded = true;

            if (t_start !== undefined && yslow_context) {
                yslow_context.PAGE.t_done = now - t_start;
                YSLOW.view.setStatusBar( yslow_context.PAGE.t_done/1000 + "s", "yslow_status_time" );
                this.panel.browser.t_start = undefined;
            }
        }

        // fire onload event.
        YSLOW.util.event.fire('onload', {'time': now, 'window': win});
        if (YSLOW.util.Preference.getPref("extensions.yslow.autorun", false)) {
            YSLOW.controller.run(win, yslow_context, true);
        }
    },

    watchTopWindow: function(win) {
        win.addEventListener("pageshow", YSLOW.firefox.onPageShow, true);
        win.addEventListener("pagehide", YSLOW.firefox.onPageHide, true);
        win.addEventListener("DOMContentLoaded", YSLOW.firefox.onDOMContentLoaded, true);

        this.win = win;
        if (typeof FirebugContext === "undefined") {
            if (this.panel === null) {
                this.panel = new YSLOW.firefox.Panel();
            }
        }
    },

    unwatchTopWindow: function(win) {
        if (this.panel) {
            this.panel.destroy();
        }
    },

    onPageShow: function(event) {
        var win = event.currentTarget;
        win.removeEventListener("pageshow", YSLOW.firefox.onPageShow, true);
        setTimeout(function() {
            YSLOW.firefox.watchLoadedTopWindow(win);
        });
    },

    onPageHide: function(event) {
        var win = event.currentTarget;
        win.removeEventListener("pagehide", YSLOW.firefox.onPageHide, true);
        if (event.persisted) {
            YSLOW.firefox.unwatchTopWindow(win);
        } else {
            win.addEventListener("unload", YSLOW.firefox.onUnload, true);
        }
    },

    onDOMContentLoaded: function(event) {
        var win = event.currentTarget;
        var now = Number(new Date());
        setTimeout(function() {
            YSLOW.util.event.fire('onDOMContentLoaded', {'time': now, 'window': win});
        });
    },

    onUnload: function(event) {
        var win = event.currentTarget;
        var now = Number(new Date());
        win.removeEventListener("unload", YSLOW.firefox.onUnload, true);
        YSLOW.firefox.unwatchTopWindow(win);

        // fire onUnload event.
        YSLOW.util.event.fire('onUnload', {'time' : now, 'window': win});

        // Clears SimpleResponseCache
        YSLOW.firefox.SimpleResponseCache.clear();

        // Clear status bar
	YSLOW.view.clearStatusBar();

	// Save the time this page UNloads, so we can determine the total load time of the NEXT page.
	// We save it in the browser object so that it is persistant ACROSS page loads, but separated
	// from one browser tab to another.
        if (YSLOW.firefox.panel) {
            YSLOW.firefox.panel.browser.t_start = now;
        }
    },

    onClickStatusIcon: function() {

        if (typeof FirebugContext !== "undefined") {
            Firebug.YSlow.onClickStatusIcon();
            return;
        }

        // toggle panel
        this.togglePanel();
    },

    onClickStatusSize: function() {
        if (typeof FirebugContext !== "undefined") {
            Firebug.YSlow.onClickStatusSize();
            return;
        }

        // toggle panel
        this.togglePanel(true);
        this.panel.doView("ysStatsButton");
    },

    onClickStatusGrade: function() {
        if (typeof FirebugContext !== "undefined") {
            Firebug.YSlow.onClickStatusGrade();
            return;
        }

        // toggle panel
        this.togglePanel(true);
        this.panel.doView("ysPerfButton");
    },

    togglePanel: function(forceOpen) {
        var panel = document.getElementById("yslow-content-box");
        var splitter = document.getElementById("yslow-content-splitter");
	splitter.collapsed = panel.collapsed = (forceOpen === undefined) ? !panel.collapsed : !forceOpen;
    },

    /**
     * Called from .xul
     */
    onToggleOption: function(menuitem) {
        var option = menuitem.getAttribute("option");
        var checked = menuitem.getAttribute("checked") == "true";
        YSLOW.util.Preference.setPref("extensions."+option, checked);
        if (option == "yslow.hidestatusbar") {
            document.getElementById("yslow-status-bar").hidden = checked;
        }
    },

    /**
     * Called from .xul
     */
    onOptionsShowing: function(popup) {
        for (var child = popup.firstChild; child; child = child.nextSibling)
        {
            if (child.localName == "menuitem")
            {
                var option = child.getAttribute("option");
                if (option)
                {
                    var checked = false;
                    checked = YSLOW.util.Preference.getPref("extensions."+option);

                    child.setAttribute("checked", checked);
                }
            }
        }
    },

    gotoYSlowHome: function() {
	gBrowser.selectedTab = gBrowser.addTab("http://developer.yahoo.com/yslow");
    },

    onPeelStart: function(event_object) {
        this.panel.createProgressBar();
    },

    onFetchDone: function(event_object) {
        this.panel.removeProgressBar();
        this.panel.doView();
    },

    showPeelProgress: function(event_object) {
        this.panel.setPeelProgress(event_object);
    },

    showFetchProgress: function(event_object) {
        this.panel.setFetchProgress(event_object);
    }

};



/**
 * Use Firefox's native networking support
 *
 * @namespace YSLOW.firefox
 * @class net
 * @static
 */
YSLOW.firefox.net = {

    /**
     * Get detail info of the passed url gathered network response from http observation notification.
     * The callback is called with an info object that includes
     * <ul>
     * <li>url</li>
     * <li>status</li>
     * <li>headers</li>
     * <li>raw_headers</li>
     * <li>body</li>
     * <li>method</li>
     * <li>type</li>
     * <li>cookie</li>
     * <li>size</li>
     * <li>respTime</li>
     * <li>startTimestamp</li>
     * </ul>
     * The callback may be called before this function returns.
     *
     * @param url
     * @param callback
     * @param binary, pass true if requesting binary content.
     * @return true if info is found, otherwise returns false.
     */
    getInfo: function(url, callback, binary) {

        var item = YSLOW.firefox.SimpleResponseCache.getItem(url);

        if (item && typeof item != "undefined") {
            if (item.raw_headers.length === 0) {
                // This may be cached data, try to get header from httpChannel before proceeding.
                YSLOW.firefox.http_observer.saveInfoFromChannel(item.httpChannel);
                item =  YSLOW.firefox.SimpleResponseCache.getItem(url);
            }

            var response = {};
            response.url = url;
            response.body = item.body;
            response.method = item.method;
            response.cookie = item.cookie;
            response.status = item.status;
            response.headers = item.headers;
            response.raw_headers = item.raw_headers;
            response.req_headers = item.req_headers;
            if (item.type !== undefined) {
                response.type = item.type;
            }
            if (item.size > 0) {
                response.size = item.size;
            }
            if (item.startTimestamp !== undefined) {
                response.startTimestamp = item.startTimestamp;
                if (item.endTimestamp !== undefined) {
                    response.respTime = item.endTimestamp - item.startTimestamp;
                }
            }

            if (item.body.length === 0 && item.headers.Location === undefined && item.httpChannel) {
                if (item.httpChannel instanceof Components.interfaces.nsICachingChannel) {
                    if (item.httpChannel.cacheKey) {
                        // Get response body here.
                        var content_loader = new YSLOW.firefox.ContentLoader(item.httpChannel, response, callback);
                        content_loader.load();
                        return true;
                    }
                }
            }
            callback(response);
            return true;
        }
        return false;

    },

    /**
     * Get url of requests identified by type.
     * @param {String|Array} type The type of component to get, e.g. "js" or ['js', 'css']
     * @return array of url
     */
    getResponseURLsByType: function(type) {

        return YSLOW.firefox.SimpleResponseCache.getURLByType(type);

    }

};

/**
 * Firefox observer
 *
 * @todo call unregister to removeObserver.
 */
YSLOW.firefox.observer = function() {
    this.register();
};

/**
 * Implement nsIObserver
 */
YSLOW.firefox.observer.prototype = {

    observe: function(subject, topic, data) {

        try {

        switch (topic) {
        case "http-on-modify-request":
        case "http-on-examine-response":
        case "http-on-examine-merged-response":
            this.handleObserverHttpTopic(subject, topic);
            break;
        default:
            return;
        }

        } catch (err) {
            YSLOW.util.dump("YSLOW.firefox.observer.observe()" + err);
        }

    },

    register: function() {

        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

        observerService.addObserver(this, "http-on-modify-request", false);
        observerService.addObserver(this, "http-on-examine-response", false);
        observerService.addObserver(this, "http-on-examine-merged-response", false);

    },

    unregister: function() {
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

        observerService.removeObserver(this, "http-on-modify-request");
        observerService.removeObserver(this, "http-on-examine-response");
        observerService.removeObserver(this, "http-on-examine-merged-response");

    },

    handleObserverHttpTopic: function(httpChannel, topic) {

        var group_observer, win, owner, obj;

        try {
            httpChannel.QueryInterface(Components.interfaces.nsIHttpChannel);
        } catch (err) {
            // can't proceed without httpChannel.
            return;
        }

        // check if this is just our response content loader
        if (httpChannel.owner) {
            try {
                owner = httpChannel.owner;
                owner.QueryInterface(Components.interfaces.nsISupportsString);
                if (owner.data == "YSLOW.firefox.ContentLoaderFlagger") {
                    // this request is generated by our content loader, don't save.
                    return;
                }
            } catch (err2) {
                // not our own request, continue.
            }
        }

        if (httpChannel.loadGroup && httpChannel.loadGroup.groupObserver) {
            group_observer = httpChannel.loadGroup.groupObserver.QueryInterface(Components.interfaces.nsIWebProgress);
            win = group_observer.DOMWindow;

            if (win !== YSLOW.firefox.win) {
                if ((httpChannel.loadFlags & Components.interfaces.nsIHttpChannel.LOAD_DOCUMENT_URI) === 0) {
                    return;
                }
            }
        }

        if (topic == "http-on-modify-request") {
            httpChannel.notificationCallbacks = this;

            obj = YSLOW.firefox.SimpleResponseCache.getItem( httpChannel.URI.asciiSpec );
            // first check if we have this url in SimpleResponseCache already.

            if (obj === null) {
                obj = {};
                obj.httpChannel = httpChannel;
                obj.status = 0;
                obj.method = httpChannel.requestMethod;
                obj.headers = {};
                obj.raw_headers = '';
                obj.body = '';
                if (httpChannel.loadFlags & Components.interfaces.nsIRequest.LOAD_BACKGROUND) {
                    obj.type = 'xhr';
                }
                // check cookie header
                try {
                    obj.cookie = httpChannel.getRequestHeader("Cookie");
                } catch (cerr) {
                    obj.cookie = '';
                }
                try {
                    var visitor = new YSLOW.firefox.HttpHeaderVisitor();
                    httpChannel.visitRequestHeaders(visitor);
                    obj.req_headers = visitor.headers;
                } catch (e) {};
                obj.size = 0;
                obj.startTimestamp = Number(new Date());
                YSLOW.firefox.SimpleResponseCache.add( httpChannel.URI.asciiSpec, obj );
            }
        } else if (topic == "http-on-examine-response" || topic == "http-on-examine-merged-response") {
            this.saveInfoFromChannel(httpChannel);
        }

    },

    saveInfoFromChannel: function(httpChannel) {

        var visitor, obj;

        visitor = new YSLOW.firefox.HttpHeaderVisitor();
        try { httpChannel.visitResponseHeaders(visitor); } catch (e) { }

        obj = YSLOW.firefox.SimpleResponseCache.getItem( httpChannel.URI.asciiSpec );
        if (obj) {
            try { obj.status = httpChannel.responseStatus; } catch (e2) { obj.status = 0; }
            obj.headers = visitor.headers;
            obj.raw_headers = visitor.raw_headers;
            try {
                if (obj.type === undefined && httpChannel.contentType) {
                    obj.type = YSLOW.util.getComponentType(httpChannel.contentType);
                }
                if (obj.status === 302) {
                    obj.type = 'redirect';
                }
            } catch(e3) {
                // don't set type.
            }
            obj.endTimestamp = Number(new Date());
            YSLOW.firefox.SimpleResponseCache.add(  httpChannel.URI.asciiSpec, obj );
        }

    },

    /*
     * nsIProgressEventSink API
     */
    onProgress: function(request, context, progress, progressMax) {

        var obj = YSLOW.firefox.SimpleResponseCache.getItem( request.name );
        if (obj) {
            obj.size = progress;
            YSLOW.firefox.SimpleResponseCache.add( request.name, obj );
        }

        return;

    },

    onStatus: function(request, context, status, statusArg) { return; },

    /*
     * nsISupports
     */
    QueryInterface: function(aIID) {
        if (aIID.equals(Components.interfaces.nsIProgressEventSink) ||
            aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
            aIID.equals(Components.interfaces.nsISupports)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    /*
     * nsIInterfaceRequestor
     */
    getInterface: function(aIID) {
        if (aIID.equals(Components.interfaces.nsIProgressEventSink)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }

};

YSLOW.firefox.HttpHeaderVisitor = function() {

    /**
     * The url of the response header being visited.
     */
    this.headers = {};
    this.raw_headers = '';

};

YSLOW.firefox.HttpHeaderVisitor.prototype = {

    visitHeader: function(name, value) {

        this.headers[name] = value;
        this.raw_headers = name + ": " + value + "\n";

    }

};

/**
 * A simple cache for responses we sniffed through observer.
 *
 * @namespace YSLOW
 * @class firefox.SimpleResponseCache
 * @static
 *
 * @todo when window content changes, SimpleResponseCache needs to be cleared.
 */
YSLOW.firefox.SimpleResponseCache = {

    cache_objs: {},

    /**
     * Add a object to the cache, index by url
     * @param url
     * @param object
     */
    add: function(url, obj) {

        this.cache_objs[url] = obj;

    },

    /**
     * Clear the cache.
     */
    clear: function() {

        this.cache_objs = [];

    },

    /**
     * Get item matching the passed url.
     * @param url
     * @return object matching the url.
     */
    getItem: function(url) {

        if (this.cache_objs[url] !== undefined) {

            return this.cache_objs[url];

        }
        return null;

    },

    /**
     * Get a list of URL by object type.
     * @param {String|Array} type The type of component to get, e.g. "js" or ['js', 'css']
     * @return list of objects having the passed type.
     */
    getURLByType: function(type) {

        var urls = [];
        var types = {};

        if (typeof type === 'string') {
            types[type] = 1;
        } else {
            for (var i in type) {
                if (type[i]) {
                    types[type[i]] = 1;
                }
            }
        }

        for (i in this.cache_objs) {
            if (this.cache_objs[i].type && typeof types[this.cache_objs[i].type] !== 'undefined') {
                urls[urls.length] = i;
            }
        }
        return urls;

    }

};

/**
 * YSLOW.firefox.ContentLoader.
 * Get response body.
 */
YSLOW.firefox.ContentLoader = function(httpChannel, response, callback) {

    this.httpChannel = httpChannel;
    this.response = response;
    this.callback = callback;

};

YSLOW.firefox.ContentLoader.prototype = {

    load: function() {

        var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var channel;

        channel = ioService.newChannel(this.response.url, null, null);
        if (channel) {
            channel.loadFlags |= Components.interfaces.nsIChannel.LOAD_FROM_CACHE | Components.interfaces.nsIChannel.LOAD_TARGETED | Components.interfaces.nsIChannel.VALIDATE_NEVER;
            channel.owner = new YSLOW.firefox.ContentLoaderFlagger();
        }

        if (channel instanceof Components.interfaces.nsICachingChannel) {
            var cacheChannel = channel;
            cacheChannel.QueryInterface(Components.interfaces.nsICachingChannel);
            cacheChannel.loadFlags |= Components.interfaces.nsICachingChannel.LOAD_ONLY_FROM_CACHE | Components.interfaces.nsIRequest.VALIDATE_NEVER;
            cacheChannel.cacheKey = this.httpChannel.cacheKey;
        }

        try
        {
            var listener = new YSLOW.firefox.ContentLoaderStreamListener(this.response.url, this);
            channel.asyncOpen(listener, null);
        }
        catch(err)
        {
            this.callback(this.response);
        }

    }

};

/**
 * YSLOW.firefox.ContentLoaderStreamListener
 */
YSLOW.firefox.ContentLoaderStreamListener = function(url, content_loader) {
    this.url = url;
    this.content_loader = content_loader;
    this.data = "";
};

YSLOW.firefox.ContentLoaderStreamListener.prototype = {

    onStartRequest: function(request, context)
    {
    },

    onStopRequest: function(request, context, status) {

        var response = this.content_loader.response;
        this.done = true;

        if (status === 0)  {
            response.body = this.data;

            // update size.  this is to work around inconsistent chunk size with image.
            var obj = YSLOW.firefox.SimpleResponseCache.getItem(response.url);
            if (response.size != obj.size) {
                response.size = obj.size;
            }
        }
        this.content_loader.callback(response);

    },

    onDataAvailable: function(request, context, input_stream, offset, count) {

        var stream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
        stream.setInputStream(input_stream);
        this.data += stream.readBytes(count);

    }

};

/**
 * YSLOW.firefox.ContentLoaderFlagger
 *
 * Owner for content loader channel.
 * With this, observer will able to distinguish the request from content loader and don't put them in SimpleCache.
 */
YSLOW.firefox.ContentLoaderFlagger = function() {
};

YSLOW.firefox.ContentLoaderFlagger.prototype = {

    data: "YSLOW.firefox.ContentLoaderFlagger",

    toString: function() {

        return "YSLOW.firefox.ContentLoaderFlagger";

    },

    QueryInterface: function(iid) {

        if (iid.equals(Components.interfaces.nsISupportsString) ||
            iid.equals(Components.interfaces.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    }
};

YSLOW.firefox.Panel = function() {
    var panel = document.getElementById("yslow-Output");
    var doc = panel.contentDocument;

    this.document = doc;
    if (panel) {
        this.initialize();
    }
};

YSLOW.firefox.Panel.prototype = {

    browser: {
        t_start: undefined
    },

    initialize: function() {
        this.panelNode = this.document.createElement("div");
        this.panelNode.ownerPanel = this;
        this.panelNode.id = "yslowDiv";
        this.panelNode.className = "panelNode panelNode-YSlow2";
        this.document.body.appendChild(this.panelNode);

        this.initializeNode();
    },

    initializeNode: function() {

        this.yslowContext = this.document.yslowContext = new YSLOW.context(window.top.content.document);

        this.ysview = new YSLOW.view(this, this.document.yslowContext);
        this.document.ysview = this.ysview;

        this.ysview.setSplashView();

    },

    destroy: function() {
        if (this.panelNode) {
            this.document.body.removeChild(this.panelNode);
            this.panelNode = undefined;
        }
    },

    getPanel: function() {
        if (this.panelNode) {
            this.destroy();
        }
        if (this.panelNode === undefined) {
            this.initialize();
        }
        return this;
    },

    createProgressBar: function() {
        this.ysview.genProgressView();
    },

    removeProgressBar: function() {
    },

    setPeelProgress: function(progress) {
        this.ysview.updateProgressView('peel', progress);
    },

    setFetchProgress: function(progress) {
        this.ysview.updateProgressView('fetch', progress);
    },

    doView: function(sView) {
        this.ysview.show(sView);
    }

};

